import type * as plugins from '../../plugins.js';

/**
 * @deprecated The legacy forwarding types are being replaced by the route-based configuration system.
 * See /ts/proxies/smart-proxy/models/route-types.ts for the new route-based configuration.
 *
 * The primary forwarding types supported by SmartProxy
 */
export type TForwardingType =
  | 'http-only'                // HTTP forwarding only (no HTTPS)
  | 'https-passthrough'        // Pass-through TLS traffic (SNI forwarding)
  | 'https-terminate-to-http'  // Terminate TLS and forward to HTTP backend
  | 'https-terminate-to-https'; // Terminate TLS and forward to HTTPS backend

/**
 * Event types emitted by forwarding handlers
 */
export enum ForwardingHandlerEvents {
  CONNECTED = 'connected',
  DISCONNECTED = 'disconnected',
  ERROR = 'error',
  DATA_FORWARDED = 'data-forwarded',
  HTTP_REQUEST = 'http-request',
  HTTP_RESPONSE = 'http-response',
  CERTIFICATE_NEEDED = 'certificate-needed',
  CERTIFICATE_LOADED = 'certificate-loaded'
}

/**
 * Base interface for forwarding handlers
 */
export interface IForwardingHandler extends plugins.EventEmitter {
  initialize(): Promise<void>;
  handleConnection(socket: plugins.net.Socket): void;
  handleHttpRequest(req: plugins.http.IncomingMessage, res: plugins.http.ServerResponse): void;
}

// Import and re-export the route-based helpers for seamless transition
import {
  createHttpRoute,
  createHttpsTerminateRoute,
  createHttpsPassthroughRoute,
  createHttpToHttpsRedirect,
  createCompleteHttpsServer,
  createLoadBalancerRoute
} from '../../proxies/smart-proxy/utils/route-helpers.js';

export {
  createHttpRoute,
  createHttpsTerminateRoute,
  createHttpsPassthroughRoute,
  createHttpToHttpsRedirect,
  createCompleteHttpsServer,
  createLoadBalancerRoute
};

/**
 * @deprecated These helper functions are maintained for backward compatibility.
 * Please use the route-based helpers instead:
 * - createHttpRoute
 * - createHttpsTerminateRoute
 * - createHttpsPassthroughRoute
 * - createHttpToHttpsRedirect
 */
import type { IRouteConfig } from '../../proxies/smart-proxy/models/route-types.js';
import { domainConfigToRouteConfig } from '../../proxies/smart-proxy/utils/route-migration-utils.js';

// For backward compatibility
export interface IForwardConfig {
  type: TForwardingType;
  target: {
    host: string | string[];
    port: number;
  };
  http?: any;
  https?: any;
  acme?: any;
  security?: any;
  advanced?: any;
  [key: string]: any;
}

export interface IDeprecatedForwardConfig {
  type: TForwardingType;
  target: {
    host: string | string[];
    port: number;
  };
  [key: string]: any;
}

/**
 * @deprecated Use createHttpRoute instead
 */
export const httpOnly = (
  partialConfig: Partial<IDeprecatedForwardConfig> & Pick<IDeprecatedForwardConfig, 'target'>
): IDeprecatedForwardConfig => ({
  type: 'http-only',
  target: partialConfig.target,
  ...(partialConfig)
});

/**
 * @deprecated Use createHttpsTerminateRoute instead
 */
export const tlsTerminateToHttp = (
  partialConfig: Partial<IDeprecatedForwardConfig> & Pick<IDeprecatedForwardConfig, 'target'>
): IDeprecatedForwardConfig => ({
  type: 'https-terminate-to-http',
  target: partialConfig.target,
  ...(partialConfig)
});

/**
 * @deprecated Use createHttpsTerminateRoute with reencrypt option instead
 */
export const tlsTerminateToHttps = (
  partialConfig: Partial<IDeprecatedForwardConfig> & Pick<IDeprecatedForwardConfig, 'target'>
): IDeprecatedForwardConfig => ({
  type: 'https-terminate-to-https',
  target: partialConfig.target,
  ...(partialConfig)
});

/**
 * @deprecated Use createHttpsPassthroughRoute instead
 */
export const httpsPassthrough = (
  partialConfig: Partial<IDeprecatedForwardConfig> & Pick<IDeprecatedForwardConfig, 'target'>
): IDeprecatedForwardConfig => ({
  type: 'https-passthrough',
  target: partialConfig.target,
  ...(partialConfig)
});